/******************************************************************************
 * @section DESCRIPTION
 *
 * Collect coupled fields and export to coupler
 *
 * Sign convention: positive value <=> downward flux
 * Units: see notes in code or seq_flds_mod.F90
 *****************************************************************************/

#include <vic_driver_cesm.h>

void
vic_cesm_put_data()
{
    extern all_vars_struct    *all_vars;
    extern force_data_struct  *force;
    extern dmy_struct          dmy_current;
    extern domain_struct       local_domain;
    extern soil_con_struct    *soil_con;
    extern veg_con_struct    **veg_con;
    extern veg_lib_struct    **veg_lib;
    extern l2x_data_struct    *l2x_vic;
    extern x2l_data_struct    *x2l_vic;
    extern global_param_struct global_param;
    extern option_struct       options;
    extern parameters_struct   param;
    extern double           ***out_data;

    bool                       IsWet = false; // TODO: add lake fraction
    bool                       overstory;
    bool                       HasVeg;
    size_t                     i;
    size_t                     veg;
    size_t                     band;
    size_t                     index;
    double                     AreaFactor;
    double                     AreaFactorSum;
    double                     TreeAdjustFactor = 1.;
    double                     lakefactor = 1.;
    double                     rad_temp;
    double                     albedo;
    double                     aero_resist;
    double                     roughness;
    double                     wind_stress;
    double                     wind_stress_x;
    double                     wind_stress_y;
    cell_data_struct           cell;
    energy_bal_struct          energy;
    snow_data_struct           snow;
    veg_var_struct             veg_var;

    for (i = 0; i < local_domain.ncells_active; i++) {
        // Zero l2x vars
        l2x_vic[i].l2x_Sl_t = 0;
        l2x_vic[i].l2x_Sl_tref = 0;
        l2x_vic[i].l2x_Sl_qref = 0;
        l2x_vic[i].l2x_Sl_avsdr = 0;
        l2x_vic[i].l2x_Sl_anidr = 0;
        l2x_vic[i].l2x_Sl_avsdf = 0;
        l2x_vic[i].l2x_Sl_anidf = 0;
        l2x_vic[i].l2x_Sl_snowh = 0;
        l2x_vic[i].l2x_Sl_u10 = 0;
        l2x_vic[i].l2x_Sl_ddvel = 0;
        l2x_vic[i].l2x_Sl_fv = 0;
        l2x_vic[i].l2x_Sl_ram1 = 0;
        l2x_vic[i].l2x_Sl_logz0 = 0;
        l2x_vic[i].l2x_Fall_taux = 0;
        l2x_vic[i].l2x_Fall_tauy = 0;
        l2x_vic[i].l2x_Fall_lat = 0;
        l2x_vic[i].l2x_Fall_sen = 0;
        l2x_vic[i].l2x_Fall_lwup = 0;
        l2x_vic[i].l2x_Fall_evap = 0;
        l2x_vic[i].l2x_Fall_swnet = 0;
        l2x_vic[i].l2x_Fall_fco2_lnd = 0;
        l2x_vic[i].l2x_Fall_flxdst1 = 0;
        l2x_vic[i].l2x_Fall_flxdst2 = 0;
        l2x_vic[i].l2x_Fall_flxdst3 = 0;
        l2x_vic[i].l2x_Fall_flxdst4 = 0;
        l2x_vic[i].l2x_Fall_flxvoc = 0;
        l2x_vic[i].l2x_Flrl_rofliq = 0;
        l2x_vic[i].l2x_Flrl_rofice = 0;

        // populate reference values

        // 10m wind, VIC: m/s, CESM: m/s
        l2x_vic[i].l2x_Sl_u10 = out_data[i][OUT_WIND][0];

        // 2m reference temperature, VIC: C, CESM: K
        l2x_vic[i].l2x_Sl_tref = out_data[i][OUT_AIR_TEMP][0] + CONST_TKFRZ;

        // 2m reference specific humidity, VIC: kg/kg, CESM: g/g
        l2x_vic[i].l2x_Sl_qref = CONST_EPS *
                                 out_data[i][OUT_VP][0] /
                                 out_data[i][OUT_PRESSURE][0];

        // band-specific quantities
        // note that these include a veg correction (AreaFactor)
        // that is already in the put_data routine

        // temperature, VIC: K, CESM: K
        l2x_vic[i].l2x_Sl_t = out_data[i][OUT_RAD_TEMP][0];

        // albedo, VIC: fraction, CESM: fraction
        // Note: VIC does not partition its albedo, thus all types are
        // the same value
        albedo = all_vars[i].gridcell_avg.avg_albedo;

        // albedo: direct, visible
        l2x_vic[i].l2x_Sl_avsdr = albedo;

        // albedo: direct , near-ir
        l2x_vic[i].l2x_Sl_anidr = albedo;

        // albedo: diffuse, visible
        l2x_vic[i].l2x_Sl_avsdf = albedo;

        // albedo: diffuse, near-ir
        l2x_vic[i].l2x_Sl_anidf = albedo;

        // snow height, VIC: mm, CESM: m
        // convert to VIC units
        l2x_vic[i].l2x_Sl_snowh = out_data[i][OUT_SWE][0] / MM_PER_M;

        // net shortwave, VIC: W/m2, CESM: W/m2
        l2x_vic[i].l2x_Fall_swnet = out_data[i][OUT_SWNET][0];

        // longwave up, VIC: W/m2, CESM: W/m2
        // adjust sign for CESM sign convention
        l2x_vic[i].l2x_Fall_lwup = -1 *
                                   (out_data[i][OUT_LWDOWN][0] -
                                    out_data[i][OUT_LWNET][0]);

        // turbulent heat fluxes
        // Note: both are the opposite sign from image driver
        // in accordance with the sign convention for coupled models
        // latent heat, VIC: W/m2, CESM: W/m2
        l2x_vic[i].l2x_Fall_lat = -1 * out_data[i][OUT_LATENT][0];

        // sensible heat, VIC: W/m2, CESM: W/m2
        l2x_vic[i].l2x_Fall_sen += -1 * out_data[i][OUT_SENSIBLE][0];

        // evaporation, VIC: mm, CESM: kg m-2 s-1
        l2x_vic[i].l2x_Fall_evap += -1 * out_data[i][OUT_EVAP][0] /
                                    global_param.dt;

        // lnd->rtm input fluxes
        l2x_vic[i].l2x_Flrl_rofliq = (out_data[i][OUT_RUNOFF][0] +
                                      out_data[i][OUT_BASEFLOW][0]) /
                                     global_param.dt;


        // running sum to make sure we get the full grid cell
        AreaFactorSum = 0;

        for (veg = 0; veg <= local_domain.locations[i].nveg; veg++) {
            overstory = veg_lib[i][veg_con[i][veg].veg_class].overstory;
            if (veg <= local_domain.locations[i].nveg - 1) {
                HasVeg = true;
            }
            else {
                HasVeg = false;
            }

            for (band = 0; band < options.SNOW_BAND; band++) {
                cell = all_vars[i].cell[veg][band];
                energy = all_vars[i].energy[veg][band];
                snow = all_vars[i].snow[veg][band];
                veg_var = all_vars[i].veg_var[veg][band];

                // TODO: Consider treeline and lake factors
                AreaFactor = (veg_con[i][veg].Cv *
                              soil_con[i].AreaFract[band] *
                              TreeAdjustFactor * lakefactor);
                if (AreaFactor < DBL_EPSILON) {
                    // Skip this patch since the area factor is zero
                    continue;
                }
                AreaFactorSum += AreaFactor;

                // aerodynamical resistance, VIC: s/m, CESM: s/m
                if (overstory) {
                    aero_resist = cell.aero_resist[1];
                }
                else {
                    aero_resist = cell.aero_resist[0];
                }

                if (aero_resist < DBL_EPSILON) {
                    log_warn("aero_resist (%f) is < %f", aero_resist,
                             DBL_EPSILON);
                    aero_resist = param.HUGE_RESIST;
                }

                // log z0
                // CESM units: m
                if (snow.snow) {
                    // snow roughness
                    roughness = soil_con[i].snow_rough;
                }
                else if (HasVeg) {
                    // vegetation roughness
                    roughness =
                        veg_lib[i][veg_con[i][veg].veg_class].roughness[
                            dmy_current.month - 1];
                }
                else {
                    // bare soil roughness
                    roughness = soil_con[i].rough;
                }
                if (roughness < DBL_EPSILON) {
                    log_warn("roughness (%f) is < %f", roughness, DBL_EPSILON);
                    roughness = DBL_EPSILON;
                }
                l2x_vic[i].l2x_Sl_logz0 += AreaFactor * log(roughness);

                // wind stress, zonal
                // CESM units: N m-2
                wind_stress_x = out_data[i][OUT_DENSITY][0] *
                                x2l_vic[i].x2l_Sa_u / aero_resist;
                l2x_vic[i].l2x_Fall_taux += AreaFactor * wind_stress_x;

                // wind stress, meridional
                // CESM units: N m-2
                wind_stress_y = out_data[i][OUT_DENSITY][0] *
                                x2l_vic[i].x2l_Sa_v / aero_resist;
                l2x_vic[i].l2x_Fall_tauy += AreaFactor * wind_stress_y;

                // friction velocity
                // CESM units: m s-1
                wind_stress =
                    sqrt(pow(wind_stress_x, 2) + pow(wind_stress_y, 2));
                l2x_vic[i].l2x_Sl_fv += AreaFactor *
                                        sqrt(wind_stress /
                                             out_data[i][OUT_DENSITY][0]);
            }
        }

        // set variables-set flag
        l2x_vic[i].l2x_vars_set = true;

        if (!assert_close_double(AreaFactorSum, 1., 0., 1e-3)) {
            log_warn("AreaFactorSum (%f) is not 1",
                     AreaFactorSum);
        }
    }
}
